@@ -18,27 +18,26 @@ from jsonbourne.core import (
 from jsonbourne.helpers import rm_js_comments
 from jsonbourne.jsonlib import import_json
 
-
 json = jsonlib
 
 JSONLIB = jsonlib.__name__
 
-__all__ = [
-    "__version__",
-    "JSON",  # js/ts JSON (THE ONE TO USE)
-    "jsonlib",  # json compat lib
-    "json",  # json compat lib
+__all__ = (
+    '__version__',
+    'JSON',  # js/ts JSON (THE ONE TO USE)
+    'jsonlib',  # json compat lib
+    'json',  # json compat lib
     # core
-    "JsonObjMutableMapping",
-    "JsonObj",
-    "JsonDict",
+    'JsonObjMutableMapping',
+    'JsonObj',
+    'JsonDict',
     # import
-    "import_json",
+    'import_json',
     # util funks
-    "stringify",
-    "parse",
-    "rm_js_comments",
+    'stringify',
+    'parse',
+    'rm_js_comments',
     # undefined
-    "undefined",
-    "UNDEFINED",
-]
+    'undefined',
+    'UNDEFINED',
+)
