@@ -5,7 +5,7 @@ import sys
 
 from functools import lru_cache
 from itertools import chain
-from json.decoder import JSONDecodeError
+from json import JSONDecodeError
 from pprint import pformat
 from types import ModuleType
 from typing import (
@@ -24,14 +24,17 @@ from typing import (
     Type,
     TypeVar,
     Union,
+    overload,
 )
 
 from jsonbourne import jsonlib
 
-
-JsonObjT = TypeVar("JsonObjT", bound='JsonObj')
+JsonPrimitiveT = TypeVar('JsonPrimitiveT', str, int, float, None)
+JsonObjT = TypeVar('JsonObjT', bound='JsonObj')
+KT = TypeVar('KT')
+VT = TypeVar('VT')
 _KT = str
-_VT = TypeVar("_VT")
+_VT = TypeVar('_VT')
 
 if sys.version_info < (3, 7):
     from collections.abc import MutableMapping
@@ -42,21 +45,21 @@ else:
 
     JsonObjMutableMapping = MutableMapping[str, _VT]
 
-__all__ = [
-    "JsonObj",
-    "JsonDict",
-    "JsonObjMutableMapping",
-    "stringify",
-    "parse",
-    "jsonify",
-    "JSON",
-    "UNDEFINED",
-    "undefined",
-    "Null",
-    "null",
-    "JSONModuleCls",
-    "JsonObjT",
-]
+__all__ = (
+    'JsonObj',
+    'JsonDict',
+    'JsonObjMutableMapping',
+    'stringify',
+    'parse',
+    'jsonify',
+    'JSON',
+    'UNDEFINED',
+    'undefined',
+    'Null',
+    'null',
+    'JSONModuleCls',
+    'JsonObjT',
+)
 
 _JsonObjMutableMapping_attrs = set(dir(JsonObjMutableMapping))
 
@@ -208,9 +211,10 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
         except AssertionError:
             d = {k: v for k, v in self._data.items() if not isinstance(k, str)}
             raise ValueError(
-                "JsonObj keys MUST be strings! Bad key values: {}".format(str(d))
+                'JsonObj keys MUST be strings! Bad key values: {}'.format(str(d))
             )
         self.recurse()
+        self.__post_init__()
 
     def recurse(self) -> None:
         """Recusively convert all sub dictionaries to JsonObj objects"""
@@ -247,8 +251,8 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
             False
 
         """
-        if "." in key:
-            first_key, _, rest = key.partition(".")
+        if '.' in key:
+            first_key, _, rest = key.partition('.')
             val = self._data.get(first_key)
             return isinstance(val, MutableMapping) and val.__contains__(rest)
         return key in self._data
@@ -293,7 +297,7 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
             return None
         if not is_identifier(key):
             raise ValueError(
-                f"Invalid key: ({key}).\n" f"Key(s) is not a valid python identifier"
+                f'Invalid key: ({key}).\n' f'Key(s) is not a valid python identifier'
             )
         self._data[key] = value
 
@@ -402,7 +406,7 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
                 return default
             raise ke
 
-    def filter_none(self, recursive: bool = False) -> "JsonObj":
+    def filter_none(self, recursive: bool = False) -> 'JsonObj':
         """Filter key-values where the value is `None` but not false-y
 
         Args:
@@ -487,7 +491,7 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
             )
         return JsonObj({k: v for k, v in self.items() if v is not None})
 
-    def filter_false(self, recursive: bool = False) -> "JsonObj":
+    def filter_false(self, recursive: bool = False) -> 'JsonObj':
         """Filter key-values where the value is false-y
 
         Args:
@@ -625,27 +629,33 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
         """
         if not isinstance(key, (str, list, tuple)):
             raise ValueError(
-                'dot_key arg must be tuple/list of strings or string; '
-                'strings will be split on \'.\''
+                ''.join(
+                    (
+                        'dot_key arg must be string or sequence of strings; ',
+                        "strings will be split on '.'",
+                    )
+                )
             )
-        parts = key.split(".") if isinstance(key, str) else list(key)
+        parts = key.split('.') if isinstance(key, str) else list(key)
         root_val: Any = self._data[parts[0]]
         cur_val = root_val
         for ix, part in enumerate(parts[1:], start=1):
             try:
                 cur_val = cur_val[part]
             except TypeError:
-                reached = ".".join(parts[:ix])
-                err_msg = f"Invalid DotKey: {key} -- Lookup reached: {reached} => {str(cur_val)}"
+                reached = '.'.join(parts[:ix])
+                err_msg = f'Invalid DotKey: {key} -- Lookup reached: {reached} => {str(cur_val)}'
                 if isinstance(key, str):
-                    err_msg += (
-                        f'\nNOTE!!! lookup performed with string (\'{key}\') '
-                        f'PREFER lookup using List[str] or Tuple[str, ...]'
+                    err_msg += ''.join(
+                        (
+                            f"\nNOTE!!! lookup performed with string ('{key}') ",
+                            'PREFER lookup using List[str] or Tuple[str, ...]',
+                        )
                     )
                 raise KeyError(err_msg)
         return cur_val
 
-    def dot_items(self) -> Iterator[Tuple[Tuple[str, ...], Any]]:
+    def dot_items(self) -> Iterator[Tuple[Tuple[str, ...], _VT]]:
         """Yield tuples of the form (dot-key, value)
 
         OG-version:
@@ -661,7 +671,7 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
                     yield k, value
         """
 
-        return chain.from_iterable(  # type: ignore
+        return chain.from_iterable(
             (
                 (
                     *(
@@ -672,10 +682,10 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
                             ),
                             dv,
                         )
-                        for dk, dv in jsonify(v).dot_items()
+                        for dk, dv in as_json_obj(v).dot_items()
                     ),
                 )
-                if isinstance(v, (JsonObj, dict)) or hasattr(v, 'dot_items')
+                if isinstance(v, (JsonObj, dict))
                 else (((str(k),), v),)
                 for k, v in self.items()
             )
@@ -688,13 +698,13 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
     def to_str(self, minify: bool = False, width: int = 88) -> str:
         """Return a string representation of the JsonObj object"""
         if minify:
-            return type(self).__name__ + "(**" + str(self.to_dict()) + ")"
-        return "".join(
+            return type(self).__name__ + '(**' + str(self.to_dict()) + ')'
+        return ''.join(
             [
                 type(self).__name__,
-                "(**{\n    ",
-                pformat(self.to_dict(), width=width)[1:-1].replace("\n", "\n   "),
-                "\n})",
+                '(**{\n    ',
+                pformat(self.to_dict(), width=width)[1:-1].replace('\n', '\n   '),
+                '\n})',
             ]
         ).replace('JsonObj(**{}),', '{},')
 
@@ -708,7 +718,7 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
 
     def _repr_html_(self) -> str:
         """Return the HTML representation of the JsonObj object"""
-        return "<pre>{}</pre>".format(self.__str__())
+        return '<pre>{}</pre>'.format(self.__str__())
 
     @classmethod
     def _cls_attr_names(cls) -> Set[str]:
@@ -716,7 +726,7 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
         try:
             return {el.name for el in cls.__attrs_attrs__}  # type: ignore
         except AttributeError:
-            raise AttributeError("Class is not decorated with attr.attrs")
+            raise AttributeError('Class is not decorated with attr.attrs')
 
     @classmethod
     def _cls_fields(cls) -> Set[str]:
@@ -724,15 +734,15 @@ class JsonObj(JsonObjMutableMapping, Generic[_VT]):
         try:
             return cls.__fields__  # type: ignore
         except AttributeError:
-            raise AttributeError("Class does not inherit from pydantic.BaseModel")
+            raise AttributeError('Class does not inherit from pydantic.BaseModel')
 
     @classmethod
     def _cls_field_names(cls) -> Set[str]:
         """Return attrs-attribute names for an object decorated with attrs"""
         try:
-            return {el for el in cls.__fields__}  # type: ignore
+            return set(cls.__fields__)  # type: ignore
         except AttributeError:
-            raise AttributeError("Class does not inherit from pydantic.BaseModel")
+            raise AttributeError('Class does not inherit from pydantic.BaseModel')
 
     @classmethod
     def _cls_protected_attrs(cls) -> Set[str]:
@@ -903,8 +913,41 @@ class JsonDict(JsonObj):
     pass
 
 
+# @overload
+# def jsonify(value: Dict[_KT, _VT]) -> JsonObj[_VT]:
+#     ...
+
+
+def as_json_obj(value: Union[JsonObj[_VT], Dict[_KT, _VT]]) -> JsonObj[_VT]:
+    if isinstance(value, dict):
+        return JsonObj(value)
+    return value
+
+
+@overload
+def jsonify(value: JsonPrimitiveT) -> JsonPrimitiveT:
+    ...
+
+
+@overload
+def jsonify(value: List[JsonPrimitiveT]) -> List[JsonPrimitiveT]:
+    ...
+
+
+@overload
+def jsonify(value: Tuple[JsonPrimitiveT, ...]) -> Tuple[JsonPrimitiveT, ...]:
+    ...
+
+
+@overload
+def jsonify(value: _VT) -> _VT:
+    ...
+
+
 def jsonify(value: Any) -> Any:
     """Convert and return a value to a JsonObj if the value is a dict"""
+    if isinstance(value, (JsonObj, JsonDict)) or issubclass(value.__class__, JsonObj):
+        return value
     if isinstance(value, dict) and not isinstance(value, JsonObj):
         return JsonObj(value)
     if isinstance(value, list):
@@ -936,7 +979,7 @@ class JSONMeta(type):
     """Meta type for use by JSON class to allow for static `__call__` method"""
 
     @staticmethod
-    def __call__(value=None):  # type: ignore
+    def __call__(value: Optional[Any] = None) -> Any:  # type: ignore
         if value is None:
             value = {}
         return jsonify(value)
@@ -962,7 +1005,11 @@ class JSON(metaclass=JSONMeta):
         """Return JSON stringified/dumps-ed data"""
         return str(
             jsonlib.dumps(
-                data, pretty=pretty, sort_keys=sort_keys, default=default, **kwargs
+                data,
+                pretty=pretty,
+                sort_keys=sort_keys,
+                default=default,
+                **kwargs,
             )
         )
 
@@ -977,7 +1024,11 @@ class JSON(metaclass=JSONMeta):
         """Return JSON stringified/dumps-ed data"""
         return str(
             jsonlib.dumps(
-                data, pretty=pretty, sort_keys=sort_keys, default=default, **kwargs
+                data,
+                pretty=pretty,
+                sort_keys=sort_keys,
+                default=default,
+                **kwargs,
             )
         )
 
@@ -992,7 +1043,11 @@ class JSON(metaclass=JSONMeta):
         """Return JSON string bytes for given data"""
         return bytes(
             jsonlib.dumpb(
-                data, pretty=pretty, sort_keys=sort_keys, default=default, **kwargs
+                data,
+                pretty=pretty,
+                sort_keys=sort_keys,
+                default=default,
+                **kwargs,
             )
         )
 
@@ -1007,7 +1062,11 @@ class JSON(metaclass=JSONMeta):
         """Return JSON string bytes for given data"""
         return bytes(
             jsonlib.dumpb(
-                data, pretty=pretty, sort_keys=sort_keys, default=default, **kwargs
+                data,
+                pretty=pretty,
+                sort_keys=sort_keys,
+                default=default,
+                **kwargs,
             )
         )
 
@@ -1076,7 +1135,6 @@ def _cls_protected_attrs(cls: Any) -> Set[str]:
 stringify = JSON.stringify
 parse = JSON.parse
 
-
 if __name__ == '__main__':
     import doctest
 
