@@ -320,7 +320,7 @@ def _import_json_stdlib() -> 'Type[JSON_STDLIB]':
 
 
 def import_json(
-    jsonlibs: Optional[Union[Tuple[str], List[str]]] = None
+    jsonlibs: Optional[Union[Tuple[str, ...], List[str]]] = None
 ) -> 'Type[JsonLibABC]':
     lib2funk = {
         'rapidjson': _import_rapidjson,
